import math
from tkinter import *
from RNGen import RNp2to191
from submodule import *


def scale_sgm( acpt_r, ck ):
    v = 1.0
    if (acpt_r > 0.5):
        ck += 1
        v = 1.0 + 2.0 * (acpt_r - 0.4) / 0.6
    elif (acpt_r < 0.3):
        ck += 1
        v = 1.0 / (1.0 + 2.0 * (0.4 - acpt_r) / 0.4)
    return v, ck

def myLog(v):
    if v < 1.0e-323:
        return -743.747
    else:
        return math.log(v)

def myExp(v):
    if v > 708.0:
        return 3.1e307
    else:
        return math.exp(v)

def LogL(Nij, Kx, Ky, Cx, Cy, rho):
    v = 0.0
    for i in range(Kx):
        for j in range(Ky):
            if i == 0:
                if j == 0:
                    #   Probs[i][j] = Pnn(Cx[0], Cy[0], rho)
                    v += Nij[i][j] * myLog(Pnn(Cx[0], Cy[0], rho))
                elif j == (Ky - 1):
                    #   Probs[i][j] = Pnp(Cx[0], Cy[K - 2], rho)
                    v += Nij[i][j] * myLog(Pnp(Cx[0], Cy[Ky - 2], rho))
                else:
                    #   Probs[i][j] = Pnc(Cx[0], Cy[j - 1], Cy[j], rho)
                    v += Nij[i][j] * myLog(Pnc(Cx[0], Cy[j - 1], Cy[j], rho))
            elif i == Kx - 1:
                if j == 0:
                    #   Probs[i][j] = Ppn(Cx[K - 2], Cy[0], rho)
                    v += Nij[i][j] * myLog(Ppn(Cx[Kx - 2], Cy[0], rho))
                elif j == Ky - 1:
                    #   Probs[i][j] = Ppp(Cx[K - 2], Cy[K - 2], rho)
                    v += Nij[i][j] * myLog(Ppp(Cx[Kx - 2], Cy[Ky - 2], rho))
                else:
                    #   Probs[i][j] = Ppc(Cx[K - 2], Cy[j - 1], Cy[j], rho)
                    v += Nij[i][j] * myLog(Ppc(Cx[Kx - 2], Cy[j - 1], Cy[j], rho))
            else:
                if j == 0:
                    #   Probs[i][j] = Pcn(Cx[i - 1], Cx[i], Cy[0], rho)
                    v += Nij[i][j] * myLog(Pcn(Cx[i - 1], Cx[i], Cy[0], rho))
                elif j == Ky - 1:
                    #   Probs[i][j] = Pcp(Cx[i - 1], Cx[i], Cy[K - 2], rho)
                    v += Nij[i][j] * myLog(Pcp(Cx[i - 1], Cx[i], Cy[Ky - 2], rho))
                else:
                    #   Probs[i][j] = Pcc(Cx[i - 1], Cx[i], Cy[j - 1], Cy[j], rho)
                    v += Nij[i][j] * myLog(Pcc(Cx[i - 1], Cx[i], Cy[j - 1], Cy[j], rho))

    return v


def mcmc_func( params ):
    """
            params = {'jump': 'Nij': 'Kx': 'Ky':
                        'Cx', 'Cy': 'NSimu': 'init_rho': 'gen_rho'}

    """
    root = Tk()
    root.title('Sampling by MCMC')
    cnvs = Canvas(root, width = 500, height = 150, background = '#FFFFFF')
    cnvs.pack()

    MsgStr = params['Msg']
    jump = params['jump']
    Nij = params['Nij']
    Kx = params['Kx']
    Ky = params['Ky']
    Cx = params['Cx']
    Cy = params['Cy']
    NSimu = params['NSimu']
    init_rho = params['init_rho']
    gen_rho = params['gen_rho']

    rhos = [init_rho]
    n_acpt = 0
    
    rn = RNp2to191()
    for i in range(0, jump):
        rn.jump(100)

    den_t = None
    ck_den = 0
    for t in range(0, NSimu):
        if (t % 100 == 0):
            print(MsgStr, '  t = ', t, '/', NSimu)
            cnvs.create_rectangle(0, 0, 500, 150, outline = '#FFFFFF', fill = '#FFFFFF')
            cnvs.create_text(250, 75,
                             text = '{0}   t = {1}/{2}'.format(MsgStr, t, NSimu),
                             font = ('', 15), fill = '#000000')

        cnvs.update()

        num = None
        den = None

        y = rn.normalMS(rhos[t], gen_rho)
        a = None
        if (math.fabs(y) >= 1.0):
            a = -1.0
        else:
            num = LogL(Nij, Kx, Ky, Cx, Cy, y)
            if ck_den == 0:
                den = LogL(Nij, Kx, Ky, Cx, Cy, rhos[t])
            else:
                den = den_t
            a = myExp(num - den)

        if rn.uni() < a:
            rhos.append(y)
            n_acpt += 1
            ck_den += 1
            den_t = num
        else:
            rhos.append(rhos[t])

    root.destroy()

    return rhos, n_acpt




